mul_montgomery_form

Function mul_montgomery_form 

pub(crate) const fn mul_montgomery_form<const LIMBS: usize>(
    a: &Uint<LIMBS>,
    b: &Uint<LIMBS>,
    modulus: &Odd<Uint<LIMBS>>,
    mod_neg_inv: Limb,
) -> Uint<LIMBS>
Computes the Montgomery product of a and b modulo modulus, where a and b are in Montgomery form.